@@ -24,7 +24,7 @@
 #include <math.h>
 #include <time.h>
 #include <string.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libplanner/mrp-project.h>
 #include <libplanner/mrp-time.h>
 #include "planner-gantt-background.h"
@@ -37,7 +37,9 @@ enum {
 	PROP_PROJECT,
 	PROP_PROJECT_START,
 	PROP_SCALE,
-	PROP_ZOOM
+	PROP_ZOOM,
+	PROP_ROW_HEIGHT,
+	PROP_SHOW_GUIDELINES
 };
 
 struct _PlannerGanttBackgroundPriv {
@@ -45,6 +47,7 @@ struct _PlannerGanttBackgroundPriv {
 	GdkGC       *fill_gc;
 	GdkGC       *timeline_gc;
 	GdkGC       *start_gc;
+	GdkGC       *guidelines_gc;
 
 	PangoLayout *layout;
 
@@ -54,9 +57,11 @@ struct _PlannerGanttBackgroundPriv {
 	MrpProject  *project;
 	MrpCalendar *calendar;
 	mrptime      project_start;
-	
+
 	gdouble      hscale;
 	gdouble      zoom;
+	gint	     row_height;
+	gboolean     show_guidelines;
 };
 
 
@@ -122,7 +127,7 @@ planner_gantt_background_get_type (void)
 					       &info,
 					       0);
 	}
-	
+
 	return type;
 }
 
@@ -183,6 +188,23 @@ gantt_background_class_init (PlannerGanttBackgroundClass *class)
 				     -G_MAXDOUBLE, G_MAXDOUBLE,
 				     7,
 				     G_PARAM_WRITABLE));
+	g_object_class_install_property (
+		gobject_class,
+		PROP_ROW_HEIGHT,
+		g_param_spec_int ("row-height",
+				  NULL,
+				  NULL,
+				  0, G_MAXINT, 0,
+				  G_PARAM_WRITABLE));
+
+	g_object_class_install_property (
+		gobject_class,
+		PROP_SHOW_GUIDELINES,
+		g_param_spec_boolean ("show-guidelines",
+				      NULL,
+				      NULL,
+				      FALSE,
+				      G_PARAM_WRITABLE));
 }
 
 static void
@@ -196,6 +218,8 @@ gantt_background_init (PlannerGanttBackground *background)
 	priv->hscale = 1.0;
 	priv->project_start = MRP_TIME_INVALID;
 	priv->timeline = mrp_time_current_time ();
+	priv->row_height = 0;
+	priv->show_guidelines = FALSE;
 }
 
 static void
@@ -208,12 +232,12 @@ gantt_background_finalize (GObject *object)
 
 	background = PLANNER_GANTT_BACKGROUND (object);
 	priv = background->priv;
-	
+
 	if (priv->timeout_id) {
 		g_source_remove (priv->timeout_id);
 		priv->timeout_id = 0;
 	}
-	
+
 	g_free (priv);
 	background->priv = NULL;
 
@@ -270,7 +294,7 @@ gantt_background_set_property (GObject      *object,
 	item = GNOME_CANVAS_ITEM (object);
 	background = PLANNER_GANTT_BACKGROUND (object);
 	priv = background->priv;
-	
+
 	switch (param_id) {
 	case PROP_PROJECT:
 		if (priv->project) {
@@ -279,7 +303,7 @@ gantt_background_set_property (GObject      *object,
 				gantt_background_project_calendar_notify_cb,
 				background);
 		}
-		
+
 		priv->project = g_value_get_object (value);
 
 		g_signal_connect (priv->project,
@@ -303,6 +327,14 @@ gantt_background_set_property (GObject      *object,
 		priv->zoom = g_value_get_double (value);
 		break;
 
+	case PROP_ROW_HEIGHT:
+		priv->row_height = g_value_get_int (value);
+		break;
+
+	case PROP_SHOW_GUIDELINES:
+		priv->show_guidelines = g_value_get_boolean (value);
+		break;
+
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -326,7 +358,7 @@ gantt_background_update (GnomeCanvasItem *item,
 							affine,
 							clip_path,
 							flags);
-	
+
 	gantt_background_get_bounds (background, &x1, &y1, &x2, &y2);
 
 	gnome_canvas_update_bbox (item, x1, y1, x2, y2);
@@ -343,7 +375,7 @@ gantt_background_realize (GnomeCanvasItem *item)
 	priv = background->priv;
 
 	GNOME_CANVAS_ITEM_CLASS (parent_class)->realize (item);
-	
+
 	priv->fill_gc = gdk_gc_new (item->canvas->layout.bin_window);
 	gnome_canvas_get_color (item->canvas, "grey96", &color);
 	gdk_gc_set_foreground (priv->fill_gc, &color);
@@ -375,14 +407,23 @@ gantt_background_realize (GnomeCanvasItem *item)
 				    GDK_CAP_BUTT,
 				    GDK_JOIN_MITER);
 
+	gnome_canvas_get_color (item->canvas, "grey80", &color);
+	priv->guidelines_gc = gdk_gc_new (item->canvas->layout.bin_window);
+	gdk_gc_set_foreground (priv->guidelines_gc, &color);
+	gdk_gc_set_line_attributes (priv->guidelines_gc,
+				    0,
+				    GDK_LINE_SOLID,
+				    GDK_CAP_BUTT,
+				    GDK_JOIN_MITER);
+
 	priv->layout = gtk_widget_create_pango_layout (GTK_WIDGET (item->canvas),
 						       NULL);
 
 	pango_layout_set_alignment (priv->layout, PANGO_ALIGN_RIGHT);
 
-	priv->timeout_id = g_timeout_add (1000 * 60,
-					  gantt_background_update_timeline,
-					  background);
+	priv->timeout_id = g_timeout_add_seconds (60,
+						  gantt_background_update_timeline,
+						  background);
 }
 
 static void
@@ -392,22 +433,28 @@ gantt_background_unrealize (GnomeCanvasItem *item)
 
 	background = PLANNER_GANTT_BACKGROUND (item);
 
-	gdk_gc_unref (background->priv->border_gc);
+	g_object_unref (background->priv->border_gc);
 	background->priv->border_gc = NULL;
 
-	gdk_gc_unref (background->priv->fill_gc);
+	g_object_unref (background->priv->fill_gc);
 	background->priv->fill_gc = NULL;
 
-	gdk_gc_unref (background->priv->timeline_gc);
+	g_object_unref (background->priv->timeline_gc);
 	background->priv->timeline_gc = NULL;
 
-	gdk_gc_unref (background->priv->start_gc);
+	g_object_unref (background->priv->start_gc);
 	background->priv->start_gc = NULL;
 
+	g_object_unref (background->priv->guidelines_gc);
+	background->priv->guidelines_gc = NULL;
+
+	g_object_unref (background->priv->layout);
+	background->priv->layout = NULL;
+
 	GNOME_CANVAS_ITEM_CLASS (parent_class)->unrealize (item);
 }
 
-static gboolean 
+static gboolean
 gantt_background_update_timeline (gpointer data)
 {
 	PlannerGanttBackground     *background;
@@ -415,11 +462,11 @@ gantt_background_update_timeline (gpointer data)
 
 	background = PLANNER_GANTT_BACKGROUND (data);
 	priv = background->priv;
-	
+
 	priv->timeline = mrp_time_current_time ();
-	
+
 	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (background));
-	
+
 	return TRUE;
 }
 
@@ -434,7 +481,7 @@ gantt_background_draw (GnomeCanvasItem *item,
 	PlannerGanttBackground     *background;
 	PlannerGanttBackgroundPriv *priv;
 	gint                   cx1, cx2;  /* Canvas pixel coordinates */
-	gint                   cy1, cy2;  
+	gint                   cy1, cy2;
 	gdouble                wx1, wx2;  /* World coordinates */
 	gdouble                hscale;
 	mrptime                t0;
@@ -445,6 +492,8 @@ gantt_background_draw (GnomeCanvasItem *item,
 	GList                 *ivals, *l;
 	MrpInterval           *ival;
 	gint                   level;
+	gdouble                i2w_dx,i2w_dy;
+	gint                   xx,yy;
 
 	background = PLANNER_GANTT_BACKGROUND (item);
 	priv = background->priv;
@@ -454,7 +503,7 @@ gantt_background_draw (GnomeCanvasItem *item,
 	}
 
 	calendar = mrp_project_get_calendar (priv->project);
-	
+
 	hscale = priv->hscale;
 	level = planner_scale_clamp_zoom (priv->zoom);
 
@@ -463,13 +512,13 @@ gantt_background_draw (GnomeCanvasItem *item,
 
 	cy1 = y;
 	cy2 = y + height;
-	
+
 	t1 = floor (wx1 / hscale + 0.5);
 	t2 = floor (wx2 / hscale + 0.5);
 
 	t0 = t1 = mrp_time_align_day (t1 - 24*60*60);
 	t2 = mrp_time_align_day (t2 + 24*60*60);
-	
+
 	/* Loop through the days between t0 and t2. */
 	while (t1 <= t2) {
 		day = mrp_calendar_get_day (calendar, t1, TRUE);
@@ -477,9 +526,9 @@ gantt_background_draw (GnomeCanvasItem *item,
 		ivals = mrp_calendar_day_get_intervals (calendar, day, TRUE);
 
 		ival_prev = t1;
-		
+
 		/* Loop through the intervals for this day. */
-		for (l = ivals; l; l = l->next) { 
+		for (l = ivals; l; l = l->next) {
 			ival = l->data;
 
 			mrp_interval_get_absolute (ival,
@@ -491,7 +540,7 @@ gantt_background_draw (GnomeCanvasItem *item,
 			 * time interval and the start of the current one,
 			 * i.e. [ival_prev, ival_start].
 			 */
-			
+
 			wx1 = ival_prev * hscale;
 			wx2 = ival_start * hscale;
 
@@ -527,10 +576,10 @@ gantt_background_draw (GnomeCanvasItem *item,
 		if (ival_prev < t1 && planner_scale_conf[level].nonworking_limit <= t1 - ival_prev) {
 			wx1 = ival_prev * hscale;
 			wx2 = t1 * hscale;
-			
+
 			gnome_canvas_w2c (item->canvas, wx1, 0, &cx1, NULL);
 			gnome_canvas_w2c (item->canvas, wx2, 0, &cx2, NULL);
-			
+
 			gdk_draw_rectangle (drawable,
 					    priv->fill_gc,
 					    TRUE,
@@ -548,12 +597,31 @@ gantt_background_draw (GnomeCanvasItem *item,
 		}
 	}
 
+	/* Guidelines */
+	if (priv->show_guidelines) {
+		i2w_dx = 0.0;
+		i2w_dy = 0.0;
+		gnome_canvas_item_i2w (item, &i2w_dx, &i2w_dy);
+		gnome_canvas_w2c (item->canvas,
+				  0 + i2w_dx,
+			  0 + i2w_dy,
+				  &xx,
+				  &yy);
+		while (yy < height+y)
+		{
+			if (yy >= y)
+				gdk_draw_line (drawable,priv->guidelines_gc,0,yy-y,width,yy-y);
+			yy += priv->row_height;
+		}
+
+	}
+
 	/* Time line for project start .*/
 	wx1 = priv->project_start * hscale;
 	gnome_canvas_w2c (item->canvas, wx1, 0, &cx1, NULL);
 
 #define DASH_LENGTH 8
-	
+
 	if (priv->project_start >= t0 && priv->project_start <= t2) {
 		gint snap;
 
@@ -569,7 +637,7 @@ gantt_background_draw (GnomeCanvasItem *item,
 			       cx1 - x,
 			       cy2 + DASH_LENGTH - y);
 	}
-	
+
 	if (priv->project_start >= t0) {
 		gchar *str, *tmp;
 		gint   label_width;
@@ -584,9 +652,9 @@ gantt_background_draw (GnomeCanvasItem *item,
 		g_free (tmp);
 
 		g_free (str);
-		
+
 		pango_layout_get_pixel_size (priv->layout, &label_width, NULL);
-		
+
 		gdk_draw_layout (drawable,
 				 GTK_WIDGET (item->canvas)->style->text_gc[GTK_STATE_NORMAL],
 				 cx1 - label_width - 5 - x,
@@ -597,11 +665,11 @@ gantt_background_draw (GnomeCanvasItem *item,
 	/* Time line for current time .*/
 	if (priv->timeline >= t0 && priv->timeline <= t2) {
 		gint snap;
-		
+
 		wx1 = priv->timeline * hscale;
 		gnome_canvas_w2c (item->canvas, wx1, 0, &cx1, NULL);
 
-		snap = cy1 - floor (cy1 / (double)DASH_LENGTH + 0.5) * DASH_LENGTH;
+		snap = cy1 - floor (cy1 / (double) DASH_LENGTH + 0.5) * DASH_LENGTH;
 
 		gdk_draw_line (drawable,
 			       priv->timeline_gc,
@@ -625,7 +693,7 @@ gantt_background_project_calendar_notify_cb (MrpProject        *project,
 					     PlannerGanttBackground *background)
 {
 	MrpCalendar *calendar;
-	
+
 	calendar = mrp_project_get_calendar (project);
 
 	gantt_background_set_calendar (background, calendar);
