@@ -19,7 +19,7 @@
  */
 
 #include <config.h>
-#include <glib.h> 
+#include <glib.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
@@ -30,14 +30,13 @@
 #include "planner-util.h"
 
 struct _PlannerPluginPriv {
-	PlannerWindow *main_window;
+	GtkActionGroup    *actions;
 };
 
 
 static void html_plugin_export (GtkAction     *action,
 				gpointer       user_data);
-void        plugin_init        (PlannerPlugin *plugin,
-				PlannerWindow *main_window);
+void        plugin_init        (PlannerPlugin *plugin);
 void        plugin_exit        (PlannerPlugin *plugin);
 
 
@@ -57,7 +56,7 @@ html_plugin_show_url (PlannerPlugin *plugin, const char *path)
 	gchar *url;
 
 	url = g_filename_to_uri (path, NULL, NULL);
-	planner_util_show_url (url, NULL);
+	planner_util_show_url (NULL, url);
 	g_free (url);
 }
 
@@ -71,10 +70,10 @@ html_plugin_export_do (PlannerPlugin *plugin,
 	GtkWidget         *dialog;
 
 	priv = plugin->priv;
-	project = planner_window_get_project (priv->main_window);
-	
+	project = planner_window_get_project (plugin->main_window);
+
 	if (!mrp_project_export (project, path, "Planner HTML", TRUE, NULL)) {
-		dialog = gtk_message_dialog_new (GTK_WINDOW (priv->main_window),
+		dialog = gtk_message_dialog_new (GTK_WINDOW (plugin->main_window),
 						 GTK_DIALOG_MODAL |
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
@@ -108,22 +107,22 @@ html_plugin_export (GtkAction *action,
 	priv = plugin->priv;
 
 	filechooser = gtk_file_chooser_dialog_new (_("Export to HTML"),
-						   GTK_WINDOW (priv->main_window),
+						   GTK_WINDOW (plugin->main_window),
 						   GTK_FILE_CHOOSER_ACTION_SAVE,
 						   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 						   GTK_STOCK_SAVE, GTK_RESPONSE_OK,
 						   NULL);
 
 	gtk_dialog_set_default_response (GTK_DIALOG (filechooser), GTK_RESPONSE_OK);
-	
-	project = planner_window_get_project (priv->main_window);
+
+	project = planner_window_get_project (plugin->main_window);
 	uri = mrp_project_get_uri (project);
 	if (!uri) {
 		gchar *cwd, *tmp;
-		
+
 		cwd = g_get_current_dir ();
 		tmp = g_strconcat (_("Unnamed"), ".html", NULL);
-		
+
 		filename = g_build_filename (cwd, tmp, NULL);
 
 		g_free (cwd);
@@ -142,7 +141,7 @@ html_plugin_export (GtkAction *action,
 
 		filename = g_strconcat (tmp, ".html", NULL);
 		g_free (tmp);
-	}		
+	}
 
 	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (filechooser),
 				       filename);
@@ -153,7 +152,7 @@ html_plugin_export (GtkAction *action,
 
 	show_button = gtk_check_button_new_with_label (_("Show result in browser"));
 	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (filechooser), show_button);
-	
+
 	g_free (basename);
 	g_free (filename);
 
@@ -175,16 +174,16 @@ html_plugin_export (GtkAction *action,
 			gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
 			res = gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
-		
+
 			switch (res) {
 			case GTK_RESPONSE_YES:
 				break;
-				
+
 			case GTK_RESPONSE_NO:
 			case GTK_RESPONSE_DELETE_EVENT:
 				g_free (filename);
 				goto try_again;
-				
+
 			default:
 				g_assert_not_reached ();
 			}
@@ -192,7 +191,7 @@ html_plugin_export (GtkAction *action,
 
 		show = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (show_button));
 		gtk_widget_destroy (filechooser);
-		
+
 		html_plugin_export_do (plugin, filename, show);
 		g_free (filename);
 
@@ -203,30 +202,26 @@ html_plugin_export (GtkAction *action,
 	}
 }
 
-G_MODULE_EXPORT void 
-plugin_init (PlannerPlugin *plugin,
-	     PlannerWindow *main_window)
+G_MODULE_EXPORT void
+plugin_init (PlannerPlugin *plugin)
 {
 	PlannerPluginPriv *priv;
 	GtkUIManager      *ui;
-	GtkActionGroup    *actions;
 	gchar             *filename;
-	
+
 	priv = g_new0 (PlannerPluginPriv, 1);
-
 	plugin->priv = priv;
-	priv->main_window = main_window;
-	
-	actions = gtk_action_group_new ("HTML plugin actions");
-	gtk_action_group_set_translation_domain (actions, GETTEXT_PACKAGE);
 
-	gtk_action_group_add_actions (actions,
+	priv->actions = gtk_action_group_new ("HTML plugin actions");
+	gtk_action_group_set_translation_domain (priv->actions, GETTEXT_PACKAGE);
+
+	gtk_action_group_add_actions (priv->actions,
 				      action_entries,
 				      G_N_ELEMENTS (action_entries),
 				      plugin);
 
-	ui = planner_window_get_ui_manager (main_window);
-	gtk_ui_manager_insert_action_group (ui, actions, 0);
+	ui = planner_window_get_ui_manager (plugin->main_window);
+	gtk_ui_manager_insert_action_group (ui, priv->actions, 0);
 
 	filename = mrp_paths_get_ui_dir ("html-plugin.ui");
 	gtk_ui_manager_add_ui_from_file (ui, filename, NULL);
@@ -235,7 +230,17 @@ plugin_init (PlannerPlugin *plugin,
 	gtk_ui_manager_ensure_update (ui);
 }
 
-G_MODULE_EXPORT void 
-plugin_exit (PlannerPlugin *plugin) 
+G_MODULE_EXPORT void
+plugin_exit (PlannerPlugin *plugin)
 {
+	PlannerPluginPriv *priv;
+	GtkUIManager      *ui;
+
+	priv = plugin->priv;
+
+	ui = planner_window_get_ui_manager (plugin->main_window);
+	gtk_ui_manager_remove_action_group (ui, priv->actions);
+	g_object_unref (priv->actions);
+
+	g_free (priv);
 }
