@@ -27,7 +27,6 @@
 #include <libplanner/mrp-resource.h>
 #include "libplanner/mrp-paths.h"
 #include "planner-usage-view.h"
-/*#include "planner-usage-print.h"*/
 #include "planner-usage-model.h"
 #include "planner-usage-tree.h"
 #include "planner-usage-chart.h"
@@ -40,13 +39,14 @@ struct _PlannerUsageViewPriv {
 
         PlannerUsageChart     *chart;
         /*PlannerUsagePrintData *print_data;*/
-	
+
 	GtkUIManager           *ui_manager;
 	GtkActionGroup         *actions;
 	guint                   merged_id;
 	gulong                  expose_id;
 };
 
+static void         usage_view_finalize                (GObject           *object);
 static void         usage_view_zoom_out_cb             (GtkAction         *action,
 							gpointer           data);
 static void         usage_view_zoom_in_cb              (GtkAction         *action,
@@ -94,13 +94,16 @@ static const gchar *usage_view_get_name                (PlannerView       *view)
 static GtkWidget *  usage_view_get_widget              (PlannerView       *view);
 static void         usage_view_print_init              (PlannerView       *view,
 							PlannerPrintJob   *job);
-static void         usage_view_print                   (PlannerView       *view);
+static void         usage_view_print                   (PlannerView       *view,
+							gint               page_nr);
 static gint         usage_view_print_get_n_pages       (PlannerView       *view);
 static void         usage_view_print_cleanup           (PlannerView       *view);
 static void         usage_view_save_columns            (PlannerUsageView  *view);
 static void         usage_view_load_columns            (PlannerUsageView  *view);
 
 
+static PlannerViewClass *parent_class = NULL;
+
 static const GtkActionEntry entries[] = {
         { "ZoomOut",   GTK_STOCK_ZOOM_OUT, N_("Zoom out"),
 	  NULL, N_("Zoom out"),
@@ -118,14 +121,51 @@ static const GtkActionEntry entries[] = {
 
 G_DEFINE_TYPE (PlannerUsageView, planner_usage_view, PLANNER_TYPE_VIEW);
 
+static gboolean
+usage_view_chart_scroll_event (GtkWidget * gki, GdkEventScroll * event, PlannerView *view)
+{
+	gboolean dontpropagate = FALSE;
+
+	gboolean can_in, can_out;
+	PlannerUsageViewPriv *priv;
+
+	if (event->state & GDK_CONTROL_MASK) {
+		priv = PLANNER_USAGE_VIEW (view)->priv;
+		planner_usage_chart_can_zoom (priv->chart, &can_in, &can_out);
+		switch (event->direction) {
+      			case GDK_SCROLL_UP: {
+				dontpropagate = TRUE;
+				if (can_in)
+					usage_view_zoom_in_cb  (NULL, view);
+	        		break;
+			}
+			case GDK_SCROLL_DOWN:
+				dontpropagate = TRUE;
+				if (can_out)
+					usage_view_zoom_out_cb  (NULL, view);
+			        break;
+		      default:
+        		break;
+		}
+    	}
+
+	return dontpropagate;
+}
+
 
 static void
 planner_usage_view_class_init (PlannerUsageViewClass *klass)
 {
+	GObjectClass     *o_class;
 	PlannerViewClass *view_class;
 
+	parent_class = g_type_class_peek_parent (klass);
+
+	o_class = (GObjectClass *) klass;
 	view_class = PLANNER_VIEW_CLASS (klass);
 
+	o_class->finalize = usage_view_finalize;
+
 	view_class->setup = usage_view_setup;
 	view_class->get_label = usage_view_get_label;
 	view_class->get_menu_label = usage_view_get_menu_label;
@@ -146,6 +186,24 @@ planner_usage_view_init (PlannerUsageView *view)
 	view->priv = g_new0 (PlannerUsageViewPriv, 1);
 }
 
+static void
+usage_view_finalize (GObject *object)
+{
+	PlannerUsageView *view;
+
+	view = PLANNER_USAGE_VIEW (object);
+
+	if (PLANNER_VIEW (view)->activated) {
+		usage_view_deactivate (PLANNER_VIEW (view));
+	}
+
+	g_free (view->priv);
+
+	if (G_OBJECT_CLASS (parent_class)->finalize) {
+		(*G_OBJECT_CLASS (parent_class)->finalize) (object);
+	}
+}
+
 static void
 usage_view_activate (PlannerView *view)
 {
@@ -179,16 +237,18 @@ usage_view_deactivate (PlannerView *view)
 
 	priv = PLANNER_USAGE_VIEW (view)->priv;
 	gtk_ui_manager_remove_ui (priv->ui_manager, priv->merged_id);
+	gtk_ui_manager_remove_action_group (priv->ui_manager, priv->actions);
+	g_object_unref (priv->actions);
+	priv->actions = NULL;
 }
 
 static void
 usage_view_setup (PlannerView *view, PlannerWindow *main_window)
 {
-        PlannerUsageViewPriv *priv;
+	PlannerUsageViewPriv *priv;
 
-        priv = g_new0 (PlannerUsageViewPriv, 1);
+	priv = PLANNER_USAGE_VIEW (view)->priv;
 
-        PLANNER_USAGE_VIEW (view)->priv = priv;
 	priv->ui_manager = planner_window_get_ui_manager(main_window);
 }
 
@@ -211,11 +271,11 @@ static const gchar *
 usage_view_get_icon (PlannerView *view)
 {
 	static gchar *filename = NULL;
-	
+
 	if (!filename) {
 		filename = mrp_paths_get_image_dir ("resources_usage.png");
 	}
-	
+
         return filename;
 }
 
@@ -250,7 +310,7 @@ usage_view_print_init (PlannerView *view, PlannerPrintJob *job)
 }
 
 static void
-usage_view_print (PlannerView *view)
+usage_view_print (PlannerView *view, gint page_nr)
 {
         /*planner_usage_print_do (PLANNER_USAGE_VIEW (view)->priv->print_data);*/
 }
@@ -310,7 +370,7 @@ usage_view_edit_columns_cb (GtkAction *action,
 {
 	PlannerUsageView     *view;
 	PlannerUsageViewPriv *priv;
-	
+
 	view = PLANNER_USAGE_VIEW (data);
 	priv = view->priv;
 
@@ -418,9 +478,14 @@ usage_view_create_widget (PlannerView *view)
         chart = planner_usage_chart_new_with_model (GTK_TREE_MODEL (model));
         priv->chart = PLANNER_USAGE_CHART (chart);
 
-	planner_usage_chart_set_view (PLANNER_USAGE_CHART (priv->chart), 
+	planner_usage_chart_set_view (PLANNER_USAGE_CHART (priv->chart),
 				      PLANNER_USAGE_TREE  (priv->tree));
 
+	gtk_widget_set_events (GTK_WIDGET (priv->chart), GDK_SCROLL_MASK);
+
+	g_signal_connect (priv->chart, "scroll-event",
+                    G_CALLBACK (usage_view_chart_scroll_event), view);
+
         sw = gtk_scrolled_window_new (hadj, vadj);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
                                         GTK_POLICY_ALWAYS,
@@ -435,12 +500,12 @@ usage_view_create_widget (PlannerView *view)
         gtk_paned_add2 (GTK_PANED (hpaned), right_frame);
 
 	usage_view_load_columns (PLANNER_USAGE_VIEW (view));
-	
+
 	g_signal_connect (tree,
 			  "columns-changed",
 			  G_CALLBACK (usage_view_tree_view_columns_changed_cb),
 			  view);
-	
+
 	g_signal_connect (tree,
 			  "destroy",
 			  G_CALLBACK (usage_view_tree_view_destroy_cb),
@@ -476,7 +541,7 @@ usage_view_create_widget (PlannerView *view)
 			  "style_set",
 			  G_CALLBACK (usage_view_tree_style_set_cb),
 			  view);
-	
+
         gtk_tree_view_expand_all (GTK_TREE_VIEW (tree));
 
 	planner_usage_chart_expand_all (PLANNER_USAGE_CHART (chart));
@@ -510,7 +575,7 @@ usage_view_update_row_and_header_height (PlannerView *view)
 
 	tv = GTK_TREE_VIEW (PLANNER_USAGE_VIEW (view)->priv->tree);
 	chart = PLANNER_USAGE_VIEW (view)->priv->chart;
-	
+
 	/* Get the row and header heights. */
 	cols = gtk_tree_view_get_columns (tv);
 	row_height = 0;
@@ -543,6 +608,8 @@ usage_view_update_row_and_header_height (PlannerView *view)
 	if (path)
 		gtk_tree_path_free (path);
 
+	g_list_free (cols);
+
 	/* Sync with the chart widget. */
 	g_object_set (chart,
 		      "header_height", header_height,
@@ -561,7 +628,7 @@ usage_view_expose_cb (GtkWidget      *widget,
 
 	g_signal_handler_disconnect (view->priv->tree,
 				     view->priv->expose_id);
-	
+
 	return FALSE;
 }
 
@@ -617,7 +684,7 @@ usage_view_project_loaded_cb (MrpProject *project, PlannerView *view)
         if (project == priv->project) {
 		/* FIXME: Due to the above, we have this hack. */
 		planner_usage_chart_setup_root_task (priv->chart);
-		
+
                 gtk_tree_view_expand_all (GTK_TREE_VIEW (priv->tree));
                 planner_usage_chart_expand_all (priv->chart);
                 return;
@@ -633,7 +700,7 @@ usage_view_project_loaded_cb (MrpProject *project, PlannerView *view)
 						  "expose_event",
 						  G_CALLBACK (usage_view_expose_cb),
 						  view);
-	
+
 	g_object_unref (model);
 
 	gtk_tree_view_expand_all (GTK_TREE_VIEW (priv->tree));
@@ -696,13 +763,13 @@ usage_view_update_zoom_sensitivity (PlannerView *view)
         gboolean              in, out;
 
 	priv = PLANNER_USAGE_VIEW (view)->priv;
-	
+
         planner_usage_chart_can_zoom (priv->chart, &in, &out);
 
 	g_object_set (gtk_action_group_get_action (
 			      GTK_ACTION_GROUP (priv->actions),
 			      "ZoomIn"),
-		      "sensitive", in, 
+		      "sensitive", in,
 		      NULL);
 
 	g_object_set (gtk_action_group_get_action (
@@ -718,7 +785,7 @@ usage_view_save_columns (PlannerUsageView *view)
 	PlannerUsageViewPriv *priv;
 
 	priv = view->priv;
-	
+
 	planner_view_column_save_helper (PLANNER_VIEW (view),
 					 GTK_TREE_VIEW (priv->tree));
 }
@@ -731,13 +798,13 @@ usage_view_load_columns (PlannerUsageView *view)
 	GtkTreeViewColumn    *column;
 	const gchar          *id;
 	gint                  i;
-		
+
 	priv = view->priv;
 
 	/* Load the columns. */
 	planner_view_column_load_helper (PLANNER_VIEW (view),
 					 GTK_TREE_VIEW (priv->tree));
-	
+
 	/* Make things a bit more robust by setting defaults if we don't get any
 	 * visible columns. Should be done through a schema instead (but we'll
 	 * keep this since a lot of people get bad installations when installing
@@ -750,7 +817,7 @@ usage_view_load_columns (PlannerUsageView *view)
 			i++;
 		}
 	}
-	
+
 	if (i == 0) {
 		for (l = columns; l; l = l->next) {
 			column = l->data;
@@ -763,11 +830,11 @@ usage_view_load_columns (PlannerUsageView *view)
 			if (!id) {
 				continue;
 			}
-			
+
 			gtk_tree_view_column_set_visible (column, TRUE);
 		}
 	}
-	
+
 	g_list_free (columns);
 }
 
